import { reactive, toRefs, ref, watch, watchEffect } from 'vue';
import { filterNoneValueObject, createParams } from './utils';
import clone from 'rfdc';
export default function useConditionWatcher(config) {
    const _conditions = reactive(config.conditions);
    const loading = ref(false);
    const data = ref(null);
    const error = ref(null);
    const refresh = ref(() => { });
    watchEffect(() => {
        const conditions2Object = { ..._conditions };
        let customConditions = {};
        const deepCopyCondition = clone({ proto: true })(conditions2Object);
        if (typeof config.beforeFetch === 'function') {
            customConditions = config.beforeFetch(deepCopyCondition);
        }
        const validateCustomConditions = Object.keys(customConditions).length !== 0;
        /*
         * if custom conditions has value, just use custom conditions
         * filterNoneValueObject will filter no value like [] , '', null, undefined
         * example. {name: '', items: [], age: 0, tags: null}
         * return result will be {age: 0}
         */
        const finalCondition = filterNoneValueObject(validateCustomConditions ? customConditions : conditions2Object);
        const params = createParams(finalCondition, config.defaultParams);
        const { loading: fetchDataLoading, result: fetchDataResult, error: fetchDataError, use: fetchData, } = useFetchData(() => config.fetcher(params));
        refresh.value = fetchData;
        loading.value = true;
        fetchData();
        watch([fetchDataResult, fetchDataError, fetchDataLoading], (values) => {
            data.value = values[0];
            error.value = values[1];
            loading.value = values[2];
        });
    });
    return {
        conditions: _conditions,
        loading,
        data,
        refresh,
        error,
    };
}
function useFetchData(fetcher) {
    const state = reactive({
        loading: false,
        error: null,
        result: null,
    });
    let lastPromise;
    const use = async () => {
        state.error = null;
        state.loading = true;
        const promise = (lastPromise = fetcher());
        try {
            const result = await promise;
            if (lastPromise === promise) {
                state.result = result;
            }
        }
        catch (e) {
            state.error = e;
        }
        finally {
            state.loading = false;
        }
    };
    return {
        ...toRefs(state),
        use,
    };
}
